--- src/hugin_base/vigra_ext/BlendPoisson.h.orig	2019-05-22 08:56:00 UTC
+++ src/hugin_base/vigra_ext/BlendPoisson.h
@@ -312,143 +312,157 @@ void SOR(Image& target, const Image& gradient, const S
         // special treatment of first line
         if (seams[0][0] > 1)
         {
-            if (doWrap)
-            {
-                const TargetPixelType delta = omega * ((gradient[0][0] + target[0][1] + 2 * target[1][0] + target[0][width - 1]) / 4.0f - target[0][0]);
-                error += detail::GetRealValue(delta*delta);
-                target[0][0] += delta;
-            }
-            else
-            {
-                const TargetPixelType delta = omega * ((gradient[0][0] + 2 * target[0][1] + 2 * target[1][0]) / 4.0f - target[0][0]);
-                error += detail::GetRealValue(delta*delta);
-                target[0][0] += delta;
-            };
+            const TargetPixelType delta = 
+                omega * (
+                    (
+                        gradient[0][0] 
+                        + target[0][1] 
+                        + 2 * target[1][0] 
+                        + (doWrap ? target[0][width - 1] : target[0][1])
+                    ) / 4.0f 
+                    - target[0][0]
+               );
+            error += detail::GetRealValue(delta*delta);
+            target[0][0] += delta;
         };
-        for (int x = 2; x < width - 1; ++(++x))
+        for (int x = 1; x < width - 1; ++x)
         {
             if (seams[0][x] > 1)
             {
-                const TargetPixelType delta = omega * ((gradient[0][x] + detail::GetBorderValues(x, 0, 1, 0, target, seams) + 2 * target[1][x]) / 4.0f - target[0][x]);
+                const TargetPixelType delta = 
+                    omega * (
+                        (
+                            gradient[0][x] 
+                            + detail::GetBorderValues(x, 0, 1, 0, target, seams) 
+                            + 2 * target[1][x]
+                        ) / 4.0f 
+                        - target[0][x]
+                    );
                 error += detail::GetRealValue(delta*delta);
                 target[0][x] += delta;
             };
         };
-        if ((width - 1) % 2 == 0 && seams[0][width - 1] > 1)
+        if (seams[0][width - 1] > 1)
         {
-            if (doWrap)
-            {
-                const TargetPixelType delta = omega * ((gradient[0][width - 1] + target[0][width - 2] + 2 * target[1][width - 1] + target[0][0]) / 4.0f - target[0][width - 1]);
-                error += detail::GetRealValue(delta*delta);
-                target[0][width - 1] += delta;
-            }
-            else
-            {
-                const TargetPixelType delta = omega * ((gradient[0][width - 1] + 2 * target[0][width - 2] + 2 * target[1][width - 1]) / 4.0f - target[0][width - 1]);
-                error += detail::GetRealValue(delta*delta);
-                target[0][width - 1] += delta;
-            };
+            const TargetPixelType delta = 
+                omega * (
+                    (
+                        gradient[0][width - 1] 
+                        + target[0][width - 2] 
+                        + 2 * target[1][width - 1] 
+                        + (doWrap ? target[0][0] : target[0][width - 2])
+                    ) / 4.0f 
+                    - target[0][width - 1]
+                );
+            error += detail::GetRealValue(delta*delta);
+            target[0][width - 1] += delta;
         };
 #pragma omp parallel for reduction(+: error) schedule(dynamic, 100)
         for (int y = 1; y < height - 1; ++y)
         {
-            if (y % 2 == 0 && seams[y][0] > 1)
+            if (seams[y][0] > 1)
             {
-                if (doWrap)
-                {
-                    const TargetPixelType delta = omega * ((gradient[y][0] + detail::GetBorderValues(0, y, 0, 1, target, seams)
-                        + target[y][1] + target[y][width - 1]) / 4.0f - target[y][0]);
-                    error += detail::GetRealValue(delta*delta);
-                    target[y][0] += delta;
-                }
-                else
-                {
-                    const TargetPixelType delta = omega * ((gradient[y][0] + detail::GetBorderValues(0, y, 0, 1, target, seams) + 2 * target[y][1]) / 4.0f - target[y][0]);
-                    error += detail::GetRealValue(delta*delta);
-                    target[y][0] += delta;
-                };
+                const TargetPixelType delta = 
+                    omega * (
+                        (
+                            gradient[y][0] 
+                            + detail::GetBorderValues(0, y, 0, 1, target, seams)
+                            + target[y][1] 
+                            + (doWrap ? target[y][width - 1] : target[y][1])
+                        ) / 4.0f 
+                        - target[y][0]
+                    );
+                error += detail::GetRealValue(delta*delta);
+                target[y][0] += delta;
             };
-            for (int x = (y % 2 == 0) ? 2 : 1; x < width - 1; ++(++x))
+            for (int x = 1; x < width - 1; ++x)
             {
                 const typename SeamMask::value_type maskValue = seams[y][x];
                 if (maskValue > 1)
                 {
-                    if (maskValue == 2)
-                    {
+                    const TargetPixelType sum = 
+                        maskValue == 2 
                         // border pixel
-                        const TargetPixelType sum = detail::GetBorderValues(x, y, 1, 0, target, seams) + detail::GetBorderValues(x, y, 0, 1, target, seams);
-                        const TargetPixelType delta = omega * ((gradient[y][x] + sum) / 4.0f - target[y][x]);
-                        error += detail::GetRealValue(delta*delta);
-                        target[y][x] += delta;
-                    }
-                    else
-                    {
-                        const TargetPixelType sum = target[y + 1][x] + target[y][x + 1] + target[y - 1][x] + target[y][x - 1];
-                        const TargetPixelType delta = omega * ((gradient[y][x] + sum) / 4.0f - target[y][x]);
-                        error += detail::GetRealValue(delta*delta);
-                        target[y][x] += delta;
-                    };
+                        ? detail::GetBorderValues(x, y, 1, 0, target, seams) 
+                          + detail::GetBorderValues(x, y, 0, 1, target, seams) 
+                        : target[y + 1][x] 
+                          + target[y][x + 1] 
+                          + target[y - 1][x] 
+                          + target[y][x - 1];
+                    const TargetPixelType delta = 
+                        omega * ((gradient[y][x] + sum) / 4.0f - target[y][x]);
+                    error += detail::GetRealValue(delta*delta);
+                    target[y][x] += delta;
                 };
             };
-            if ((y + width - 1) % 2 == 0 && seams[y][width - 1] > 1)
+            if (seams[y][width - 1] > 1)
             {
-                if (doWrap)
-                {
-                    const TargetPixelType delta = omega * ((gradient[y][width - 1] + detail::GetBorderValues(width - 1, y, 0, 1, target, seams) + target[y][width - 2] + target[y][0]) / 4.0f - target[y][width - 1]);
-                    error += detail::GetRealValue(delta*delta);
-                    target[y][width - 1] += delta;
-                }
-                else
-                {
-                    const TargetPixelType delta = omega * ((gradient[y][width - 1] + detail::GetBorderValues(width - 1, y, 0, 1, target, seams) + 2 * target[y][width - 2]) / 4.0f - target[y][width - 1]);
-                    error += detail::GetRealValue(delta*delta);
-                    target[y][width - 1] += delta;
-                };
+                const TargetPixelType delta = 
+                    omega * (
+                        (
+                            gradient[y][width - 1] 
+                            + detail::GetBorderValues(width - 1, y, 0, 1, target, seams) 
+                            + target[y][width - 2] 
+                            + (doWrap ? target[y][0] : target[y][width - 2])
+                        ) / 4.0f 
+                        - target[y][width - 1]
+                    );
+                error += detail::GetRealValue(delta*delta);
+                target[y][width - 1] += delta;
             };
         };
         // last line
-        if ((height - 1) % 2 == 0 && seams[height - 1][0] > 1)
+        if (seams[height - 1][0] > 1)
         {
-            if (doWrap)
-            {
-                const TargetPixelType delta = omega * ((gradient[height - 1][0] + 2 * target[height - 2][0] + target[height - 1][1] + target[height - 1][width - 1]) / 4.0f - target[height - 1][0]);
-                error += detail::GetRealValue(delta*delta);
-                target[height - 1][0] += delta;
-            }
-            else
-            {
-                const TargetPixelType delta = omega * ((gradient[height - 1][0] + 2 * target[height - 2][0] + 2 * target[height - 1][1]) / 4.0f - target[height - 1][0]);
-                error += detail::GetRealValue(delta*delta);
-                target[height - 1][0] += delta;
-            };
+            const TargetPixelType delta = 
+                omega * (
+                    (
+                        gradient[height - 1][0] 
+                        + 2 * target[height - 2][0] 
+                        + target[height - 1][1] 
+                        + (doWrap ? target[height - 1][width - 1] : target[height - 1][1])
+                    ) / 4.0f 
+                    - target[height - 1][0]
+                );
+            error += detail::GetRealValue(delta*delta);
+            target[height - 1][0] += delta;
         };
-        for (int x = (height - 1) % 2 == 0 ? 2 : 1; x < width - 1; ++(++x))
+        for (int x = 1; x < width - 1; ++x)
         {
             if (seams[height - 1][x] > 1)
             {
-                const TargetPixelType delta = omega * ((gradient[height - 1][x] + detail::GetBorderValues(x, height - 1, 1, 0, target, seams) + 2 * target[height - 2][x]) / 4.0f - target[height - 1][x]);
+                const TargetPixelType delta = 
+                    omega * (
+                        (
+                            gradient[height - 1][x] 
+                            + detail::GetBorderValues(x, height - 1, 1, 0, target, seams) 
+                            + 2 * target[height - 2][x]
+                        ) / 4.0f 
+                        - target[height - 1][x]
+                    );
                 error += detail::GetRealValue(delta*delta);
                 target[height - 1][x] += delta;
             };
         };
-        if ((width - 1 + height - 1) % 2 == 0 && seams[height - 1][width - 1] > 1)
+        if (seams[height - 1][width - 1] > 1)
         {
-            if (doWrap)
-            {
-                const TargetPixelType delta = omega * ((gradient[height - 1][width - 1] + 2 * target[height - 2][width - 1] + target[height - 1][width - 2] + target[height - 1][0]) / 4.0f - target[height - 1][width - 1]);
-                error += detail::GetRealValue(delta*delta);
-                target[height - 1][width - 1] += delta;
-            }
-            else
-            {
-                const TargetPixelType delta = omega * ((gradient[height - 1][width - 1] + 2 * target[height - 2][width - 1] + 2 * target[height - 1][width - 2]) / 4.0f - target[height - 1][width - 1]);
-                error += detail::GetRealValue(delta*delta);
-                target[height - 1][width - 1] += delta;
-            };
+            const TargetPixelType delta = 
+                omega * (
+                    (
+                        gradient[height - 1][width - 1] 
+                        + 2 * target[height - 2][width - 1] 
+                        + target[height - 1][width - 2] 
+                        + (doWrap ? target[height - 1][0] : target[height - 1][width - 2])
+                    ) / 4.0f 
+                    - target[height - 1][width - 1]
+                );
+            error += detail::GetRealValue(delta*delta);
+            target[height - 1][width - 1] += delta;
         };
+#if 0
         // now red pattern
         // special treatment of first line
-        for (int x = 1; x < width - 1; ++(++x))
+        for (int x = 1; x < width - 1; ++x)
         {
             if (seams[0][x] > 1)
             {
@@ -459,116 +473,122 @@ void SOR(Image& target, const Image& gradient, const S
         };
         if ((width - 1) % 2 == 1 && seams[0][width - 1] > 1)
         {
-            if (doWrap)
-            {
-                const TargetPixelType delta = omega * ((gradient[0][width - 1] + target[0][width - 2] + 2 * target[1][width - 1] + target[0][0]) / 4.0f - target[0][width - 1]);
-                error += detail::GetRealValue(delta*delta);
-                target[0][width - 1] += delta;
-            }
-            else
-            {
-                const TargetPixelType delta = omega * ((gradient[0][width - 1] + 2 * target[0][width - 2] + 2 * target[1][width - 1]) / 4.0f - target[0][width - 1]);
-                error += detail::GetRealValue(delta*delta);
-                target[0][width - 1] += delta;
-            };
+            const TargetPixelType delta = 
+                omega * (
+                    (
+                        gradient[0][width - 1] 
+                        + target[0][width - 2] 
+                        + 2 * target[1][width - 1] 
+                        + (doWrap ? target[0][0] : target[0][width - 2])
+                    ) / 4.0f 
+                    - target[0][width - 1]
+                );
+            error += detail::GetRealValue(delta*delta);
+            target[0][width - 1] += delta;
         };
 #pragma omp parallel for reduction(+: error) schedule(dynamic, 100)
         for (int y = 1; y < height - 1; ++y)
         {
             if (y % 2 == 1 && seams[y][0] > 1)
             {
-                if (doWrap)
-                {
-                    const TargetPixelType delta = omega * ((gradient[y][0] + detail::GetBorderValues(0, y, 0, 1, target, seams)
-                        + target[y][1] + target[y][width - 1]) / 4.0f - target[y][0]);
-                    error += detail::GetRealValue(delta*delta);
-                    target[y][0] += delta;
-                }
-                else
-                {
-                    const TargetPixelType delta = omega * ((gradient[y][0] + detail::GetBorderValues(0, y, 0, 1, target, seams) + 2 * target[y][1]) / 4.0f - target[y][0]);
-                    error += detail::GetRealValue(delta*delta);
-                    target[y][0] += delta;
-                };
+                const TargetPixelType delta = 
+                    omega * (
+                        (
+                            gradient[y][0] 
+                            + detail::GetBorderValues(0, y, 0, 1, target, seams)
+                            + target[y][1] 
+                            + (doWrap ? target[y][width - 1] : target[y][1])
+                        ) / 4.0f 
+                        - target[y][0]
+                    );
+                error += detail::GetRealValue(delta*delta);
+                target[y][0] += delta;
             };
             for (int x = (y % 2 == 1) ? 2 : 1; x < width - 1; ++(++x))
             {
                 const typename SeamMask::value_type maskValue = seams[y][x];
                 if (maskValue > 1)
                 {
-                    if (maskValue == 2)
-                    {
+                    const TargetPixelType sum = 
+                        maskValue == 2 
                         // border pixel
-                        const TargetPixelType sum = detail::GetBorderValues(x, y, 1, 0, target, seams) + detail::GetBorderValues(x, y, 0, 1, target, seams);
-                        const TargetPixelType delta = omega * ((gradient[y][x] + sum) / 4.0f - target[y][x]);
-                        error += detail::GetRealValue(delta*delta);
-                        target[y][x] += delta;
-                    }
-                    else
-                    {
-                        const TargetPixelType sum = target[y + 1][x] + target[y][x + 1] + target[y - 1][x] + target[y][x - 1];
-                        const TargetPixelType delta = omega * ((gradient[y][x] + sum) / 4.0f - target[y][x]);
-                        error += detail::GetRealValue(delta*delta);
-                        target[y][x] += delta;
-                    };
+                        ? detail::GetBorderValues(x, y, 1, 0, target, seams) 
+                          + detail::GetBorderValues(x, y, 0, 1, target, seams) 
+                        : target[y + 1][x] 
+                          + target[y][x + 1] 
+                          + target[y - 1][x] 
+                          + target[y][x - 1];
+                    const TargetPixelType delta = 
+                        omega * ((gradient[y][x] + sum) / 4.0f - target[y][x]);
+                    error += detail::GetRealValue(delta*delta);
+                    target[y][x] += delta;
                 };
             };
             if ((y + width - 1) % 2 == 1 && seams[y][width - 1] > 1)
             {
-                if (doWrap)
-                {
-                    const TargetPixelType delta = omega * ((gradient[y][width - 1] + detail::GetBorderValues(width - 1, y, 0, 1, target, seams) + target[y][width - 2] + target[y][0]) / 4.0f - target[y][width - 1]);
-                    error += detail::GetRealValue(delta*delta);
-                    target[y][width - 1] += delta;
-                }
-                else
-                {
-                    const TargetPixelType delta = omega * ((gradient[y][width - 1] + detail::GetBorderValues(width - 1, y, 0, 1, target, seams) + 2 * target[y][width - 2]) / 4.0f - target[y][width - 1]);
-                    error += detail::GetRealValue(delta*delta);
-                    target[y][width - 1] += delta;
-                };
+                const TargetPixelType delta = 
+                    omega * (
+                        (
+                            gradient[y][width - 1] 
+                            + detail::GetBorderValues(width - 1, y, 0, 1, target, seams) 
+                            + target[y][width - 2] 
+                            + (doWrap ? target[y][0] : target[y][width - 2])
+                        ) / 4.0f 
+                        - target[y][width - 1]
+                    );
+                error += detail::GetRealValue(delta*delta);
+                target[y][width - 1] += delta;
             };
         };
         // last line
         if ((height - 1) % 2 == 1 && seams[height - 1][0] > 1)
         {
-            if (doWrap)
-            {
-                const TargetPixelType delta = omega * ((gradient[height - 1][0] + 2 * target[height - 2][0] + target[height - 1][1] + target[height - 1][width - 1]) / 4.0f - target[height - 1][0]);
-                error += detail::GetRealValue(delta*delta);
-                target[height - 1][0] += delta;
-            }
-            else
-            {
-                const TargetPixelType delta = omega * ((gradient[height - 1][0] + 2 * target[height - 2][0] + 2 * target[height - 1][1]) / 4.0f - target[height - 1][0]);
-                error += detail::GetRealValue(delta*delta);
-                target[height - 1][0] += delta;
-            };
+            const TargetPixelType delta = 
+                omega * (
+                    (
+                        gradient[height - 1][0] 
+                        + 2 * target[height - 2][0] 
+                        + target[height - 1][1] 
+                        + (doWrap ? target[height - 1][width - 1] : target[height - 1][1])
+                    ) / 4.0f 
+                    - target[height - 1][0]
+                );
+            error += detail::GetRealValue(delta*delta);
+            target[height - 1][0] += delta;
         };
         for (int x = (height - 1) % 2 == 1 ? 2 : 1; x < width - 1; ++(++x))
         {
             if (seams[height - 1][x] > 1)
             {
-                const TargetPixelType delta = omega * ((gradient[height - 1][x] + detail::GetBorderValues(x, height - 1, 1, 0, target, seams) + 2 * target[height - 2][x]) / 4.0f - target[height - 1][x]);
+                const TargetPixelType delta = 
+                    omega * (
+                        (
+                            gradient[height - 1][x] 
+                            + detail::GetBorderValues(x, height - 1, 1, 0, target, seams) 
+                            + 2 * target[height - 2][x]
+                        ) / 4.0f 
+                        - target[height - 1][x]
+                    );
                 error += detail::GetRealValue(delta*delta);
                 target[height - 1][x] += delta;
             };
         };
         if ((width - 1 + height - 1) % 2 == 1 && seams[height - 1][width - 1] > 1)
         {
-            if (doWrap)
-            {
-                const TargetPixelType delta = omega * ((gradient[height - 1][width - 1] + 2 * target[height - 2][width - 1] + target[height - 1][width - 2] + target[height - 1][0]) / 4.0f - target[height - 1][width - 1]);
-                error += detail::GetRealValue(delta*delta);
-                target[height - 1][width - 1] += delta;
-            }
-            else
-            {
-                const TargetPixelType delta = omega * ((gradient[height - 1][width - 1] + 2 * target[height - 2][width - 1] + 2 * target[height - 1][width - 2]) / 4.0f - target[height - 1][width - 1]);
-                error += detail::GetRealValue(delta*delta);
-                target[height - 1][width - 1] += delta;
-            };
+            const TargetPixelType delta = 
+                omega * (
+                    (
+                        gradient[height - 1][width - 1] 
+                        + 2 * target[height - 2][width - 1] 
+                        + target[height - 1][width - 2] 
+                        + (doWrap ? target[height - 1][0] : target[height - 1][width - 2])
+                    ) / 4.0f 
+                    - target[height - 1][width - 1]
+                );
+            error += detail::GetRealValue(delta*delta);
+            target[height - 1][width - 1] += delta;
         };
+#endif
 
         if (oldError > 0 && log(oldError / error) / log(10.0) < errorThreshold)
         {
@@ -587,120 +607,99 @@ void CalcResidualError(Image& error, const Image& targ
 
     if (seam[0][0] > 1)
     {
-        if (doWrap)
-        {
-            const ImagePixelType sum = 2 * target[1][0] + target[0][1] + target[0][width - 1];
-            error[0][0] = (4 * target[0][0] - sum - gradient[0][0]);
-        }
-        else
-        {
-            const ImagePixelType sum = 2 * target[1][0] + 2 * target[0][1];
-            error[0][0] = (4 * target[0][0] - sum - gradient[0][0]);
-        };
+        const ImagePixelType sum = 
+            2 * target[1][0] 
+            + target[0][1] 
+            + (doWrap ? target[0][width - 1] : target[0][1]);
+        error[0][0] = 4 * target[0][0] - sum - gradient[0][0];
     };
     for (int x = 1; x < width - 1; ++x)
     {
-        if (seam[0][x]>1)
+        if (seam[0][x] > 1)
         {
-            const ImagePixelType sum = 2 * target[1][x] + detail::GetBorderValues(x, 0, 1, 0, target, seam);
-            error[0][x] = (4 * target[0][x] - sum - gradient[0][x]);
+            const ImagePixelType sum = 
+                2 * target[1][x] 
+                + detail::GetBorderValues(x, 0, 1, 0, target, seam);
+            error[0][x] = 4 * target[0][x] - sum - gradient[0][x];
         };
     };
     if (seam[0][width - 1] > 1)
     {
-        if (doWrap)
-        {
-            const ImagePixelType sum = 2 * target[1][width - 1] + target[0][width - 2] + target[0][0];
-            error[0][width - 1] = (4 * target[0][width - 1] - sum - gradient[0][width - 1]);
-        }
-        else
-        {
-            const ImagePixelType sum = 2 * target[1][width - 1] + 2 * target[0][width - 2];
-            error[0][width - 1] = (4 * target[0][width - 1] - sum - gradient[0][width - 1]);
-        }
+        const ImagePixelType sum = 
+            2 * target[1][width - 1] 
+            + target[0][width - 2] 
+            + (doWrap ? target[0][0] : target[0][width - 2]);
+        error[0][width - 1] = 
+            4 * target[0][width - 1] - sum - gradient[0][width - 1];
     };
 #pragma omp parallel for schedule(dynamic, 100)
     for (int y = 1; y < height - 1; ++y)
     {
         if (seam[y][0] > 1)
         {
-            if (doWrap)
-            {
-                const ImagePixelType sum = detail::GetBorderValues(0, y, 0, 1, target, seam) + target[y][1] + target[y][width - 1];
-                error[y][0] = (4 * target[y][0] - sum - gradient[y][0]);
-            }
-            else
-            {
-                const ImagePixelType sum = detail::GetBorderValues(0, y, 0, 1, target, seam) + 2 * target[y][1];
-                error[y][0] = (4 * target[y][0] - sum - gradient[y][0]);
-            };
+            const ImagePixelType sum = 
+                detail::GetBorderValues(0, y, 0, 1, target, seam) 
+                + target[y][1] 
+                + (doWrap ? target[y][width - 1] : target[y][1]);
+            error[y][0] = 4 * target[y][0] - sum - gradient[y][0];
         }
         for (int x = 1; x < width - 1; ++x)
         {
             const typename SeamMask::value_type maskValue = seam[y][x];
             if (maskValue > 1)
             {
-                if (maskValue == 2)
-                {
+                const ImagePixelType sum = 
+                    maskValue == 2 
                     // border pixel
-                    const ImagePixelType sum = detail::GetBorderValues(x, y, 1, 0, target, seam) + detail::GetBorderValues(x, y, 0, 1, target, seam);
-                    error[y][x] = (4 * target[y][x] - sum - gradient[y][x]);
-                }
-                else
-                {
-                    const ImagePixelType sum = target[y + 1][x] + target[y][x + 1] + target[y - 1][x] + target[y][x - 1];
-                    error[y][x] = (4 * target[y][x] - sum - gradient[y][x]);
-                };
+                    ? detail::GetBorderValues(x, y, 1, 0, target, seam) 
+                      + detail::GetBorderValues(x, y, 0, 1, target, seam) 
+                    : target[y + 1][x] 
+                      + target[y][x + 1] 
+                      + target[y - 1][x] 
+                      + target[y][x - 1];
+                error[y][x] = 4 * target[y][x] - sum - gradient[y][x];
             };
         };
         if (seam[y][width - 1] > 1)
         {
-            if (doWrap)
-            {
-                const ImagePixelType sum = detail::GetBorderValues(width - 1, y, 0, 1, target, seam) + target[y][width - 2] + target[y][0];
-                error[y][width - 1] = (4 * target[y][width - 1] - sum - gradient[y][width - 1]);
-            }
-            else
-            {
-                const ImagePixelType sum = detail::GetBorderValues(width - 1, y, 0, 1, target, seam) + 2 * target[y][width - 2];
-                error[y][width - 1] = (4 * target[y][width - 1] - sum - gradient[y][width - 1]);
-            };
+            const ImagePixelType sum = 
+                detail::GetBorderValues(width - 1, y, 0, 1, target, seam) 
+                + target[y][width - 2] 
+                + (doWrap ? target[y][0] : target[y][width - 2]);
+            error[y][width - 1] = 
+                4 * target[y][width - 1] - sum - gradient[y][width - 1];
         };
     };
     // last line
     if (seam[height - 1][0] > 1)
     {
-        if (doWrap)
-        {
-            const ImagePixelType sum = 2 * target[height - 2][0] + target[height - 1][width - 1] + target[height - 1][1];
-            error[height - 1][0] = (4 * target[height - 1][0] - sum - gradient[height - 1][0]);
-        }
-        else
-        {
-            const ImagePixelType sum = 2 * target[height - 2][0] + 2 * target[height - 1][1];
-            error[height - 1][0] = (4 * target[height - 1][0] - sum - gradient[height - 1][0]);
-        };
+        const ImagePixelType sum = 
+            2 * target[height - 2][0] 
+            + target[height - 1][1] 
+            + (doWrap ? target[height - 1][width - 1] : target[height - 1][1]);
+        error[height - 1][0] = 
+            4 * target[height - 1][0] - sum - gradient[height - 1][0];
     };
     for (int x = 1; x < width - 1; ++x)
     {
-        if (seam[height - 1][x]>1)
+        if (seam[height - 1][x] > 1)
         {
-            const ImagePixelType sum = 2 * target[height-2][x] + detail::GetBorderValues(x, height - 1, 1, 0, target, seam);
-            error[height - 1][x] = (4 * target[height - 1][x] - sum - gradient[height - 1][x]);
+            const ImagePixelType sum = 
+                2 * target[height-2][x] 
+                + detail::GetBorderValues(x, height - 1, 1, 0, target, seam);
+            error[height - 1][x] = 
+                4 * target[height - 1][x] - sum - gradient[height - 1][x];
         };
     };
     if (seam[height - 1][width - 1] > 1)
     {
-        if (doWrap)
-        {
-            const ImagePixelType sum = 2 * target[height - 2][width - 1] + target[height - 1][width - 2] + target[height - 1][0];
-            error[height - 1][width - 1] = (4 * target[height - 1][width - 1] - sum - gradient[height - 1][width - 1]);
-        }
-        else
-        {
-            const ImagePixelType sum = 2 * target[height - 2][width - 1] + 2 * target[height - 1][width - 2];
-            error[height - 1][width - 1] = (4 * target[height - 1][width - 1] - sum - gradient[height - 1][width - 1]);
-        };
+        const ImagePixelType sum = 
+            2 * target[height - 2][width - 1] 
+            + target[height - 1][width - 2] 
+            + (doWrap ? target[height - 1][0] : target[height - 1][width - 2]);
+        error[height - 1][width - 1] = 
+            4 * target[height - 1][width - 1] - sum 
+            - gradient[height - 1][width - 1];
     };
 
 }
