--- src/tools/hugin_hdrmerge.cpp.orig	2017-04-05 15:29:09 UTC
+++ src/tools/hugin_hdrmerge.cpp
@@ -204,8 +204,9 @@ static void usage(const char* name)
 int main(int argc, char* argv[])
 {
 
+    try {
     // parse arguments
-    const char* optstring = "chvo:m:i:s:a:el";
+    const char* optstring = "chvo:m:i:s:a:elB:M:";
     static struct option longOptions[] =
     {
         { "output", required_argument, NULL, 'o' },
@@ -265,6 +266,24 @@ int main(int argc, char* argv[])
             case 'v':
                 g_verbose++;
                 break;
+            case 'B': {
+                const int kilobytes = atoi(optarg);
+                if (kilobytes < 1) {
+                    std::cerr << "cache block size must be 1 or more." << std::endl;
+                    return 1;
+                }
+                //CachedFileImageDirector::v().setBlockSize(static_cast<long long>(kilobytes) << 10);
+                break;
+            }
+            case 'M': {
+                const int megabytes = atoi(optarg);
+                if (megabytes < 1) {
+                    std::cerr << "memory limit must be 1 or more." << std::endl;
+                    return 1;
+                }
+                //CachedFileImageDirector::v().setAllocation(static_cast<long long>(megabytes) << 20);
+                break;
+            }
             case 'h':
                 usage(hugin_utils::stripPath(argv[0]).c_str());
                 return 0;
@@ -304,8 +323,6 @@ int main(int argc, char* argv[])
 
     // output image
     ImageType output;
-    try
-    {
         if (mode == "avg_slow")
         {
             // use a weighted average, with special consideration of pixels
@@ -323,7 +340,25 @@ int main(int argc, char* argv[])
             }
             vigra::ImageExportInfo exinfo(outputFile.c_str());
             exinfo.setPixelType("FLOAT");
+            int c = 0, a = 0;
             vigra::BImage alpha(output.width(), output.height(), 255);
+            for (size_t j = output.height(); --j >= 0; ) {
+                for (size_t i = output.width(); --i >= 0; ) {
+                    ++c;
+                    vigra::RGBValue<float> rgb = output(i,j);
+                    if (::isnan(rgb[vigra::RGBValue<float>::RedIdx]) 
+                        || ::isnan(rgb[vigra::RGBValue<float>::GreenIdx]) 
+                        || ::isnan(rgb[vigra::RGBValue<float>::BlueIdx])) {
+                        ++a;
+                        alpha(i,j) = 0;
+                        rgb[vigra::RGBValue<float>::RedIdx] = 
+                           rgb[vigra::RGBValue<float>::GreenIdx] = 
+                           rgb[vigra::RGBValue<float>::BlueIdx] = 0.;
+                        output(i,j) = rgb;
+                    }
+                }
+            }
+            std::cerr << c << " pixels total; " << a << " pixels transparent" << std::endl;
             vigra::exportImageAlpha(srcImageRange(output), srcImage(alpha), exinfo);
         }
         else if (mode == "avg")
@@ -367,11 +402,12 @@ int main(int argc, char* argv[])
             std::cerr << "Unknown merge mode, see help for a list of possible modes" << std::endl;
             return 1;
         }
-    }
-    catch (std::exception& e)
-    {
+    } catch (const std::exception &e) {
         std::cerr << "caught exception: " << e.what() << std::endl;
-        abort();
+        return 1;
+    } catch (...) {
+        std::cerr << "caught unknown exception" << std::endl;
+        return 1;
     }
 
     return 0;
