--- src/hugin1/hugin/ImagesTree.cpp.orig	2020-01-08 14:35:27 UTC
+++ src/hugin1/hugin/ImagesTree.cpp
@@ -53,6 +53,8 @@ enum
     ID_UNSELECT_ALL,
     ID_SELECT_LENS_STACK,
     ID_UNSELECT_LENS_STACK,
+    ID_SELECT_SELECTED,
+    ID_UNSELECT_SELECTED,
     ID_ACTIVATE_IMAGE,
     ID_DEACTIVATE_IMAGE,
     ID_OPERATION_START=wxID_HIGHEST+300
@@ -69,6 +71,8 @@ BEGIN_EVENT_TABLE(ImagesTreeCtrl, wxTreeListCtrl)
     EVT_MENU(ID_UNSELECT_ALL, ImagesTreeCtrl::OnUnselectAll)
     EVT_MENU(ID_SELECT_LENS_STACK, ImagesTreeCtrl::OnSelectLensStack)
     EVT_MENU(ID_UNSELECT_LENS_STACK, ImagesTreeCtrl::OnUnselectLensStack)
+    EVT_MENU(ID_SELECT_SELECTED, ImagesTreeCtrl::OnSelectSelected)
+    EVT_MENU(ID_UNSELECT_SELECTED, ImagesTreeCtrl::OnUnselectSelected)
     EVT_MENU(ID_ACTIVATE_IMAGE, ImagesTreeCtrl::OnActivateImage)
     EVT_MENU(ID_DEACTIVATE_IMAGE, ImagesTreeCtrl::OnDeactivateImage)
     EVT_MENU_RANGE(ID_OPERATION_START, ID_OPERATION_START+50, ImagesTreeCtrl::OnExecuteOperation)
@@ -176,9 +180,9 @@ void ImagesTreeCtrl::CreateColumns()
 
     ADDCOLUMN(_("Lens type (f)"), "projection", 100, wxALIGN_LEFT, false, HuginBase::ImageVariableGroup::IVE_Filename, _("Lens type (rectilinear, fisheye, equirectangular, ...)"))
     ADDCOLUMN(_("Hfov (v)"), "v", 80, wxALIGN_LEFT, true, HuginBase::ImageVariableGroup::IVE_HFOV, _("Horizontal field of view (v)"))
-    ADDCOLUMN(wxT("a"), "a", 40, wxALIGN_LEFT, true, HuginBase::ImageVariableGroup::IVE_RadialDistortion, _("Radial distortion (a)"))
-    ADDCOLUMN(wxT("b"), "b", 40, wxALIGN_LEFT, true, HuginBase::ImageVariableGroup::IVE_RadialDistortion, _("Radial distortion (b, barrel)"))
-    ADDCOLUMN(wxT("c"), "c", 40, wxALIGN_LEFT, true, HuginBase::ImageVariableGroup::IVE_RadialDistortion, _("Radial distortion (c)"))
+    ADDCOLUMN(wxT("a"), "a", 40, wxALIGN_LEFT, true, HuginBase::ImageVariableGroup::IVE_RadialDistortionA, _("Radial distortion (a)"))
+    ADDCOLUMN(wxT("b"), "b", 40, wxALIGN_LEFT, true, HuginBase::ImageVariableGroup::IVE_RadialDistortionB, _("Radial distortion (b, barrel)"))
+    ADDCOLUMN(wxT("c"), "c", 40, wxALIGN_LEFT, true, HuginBase::ImageVariableGroup::IVE_RadialDistortionC, _("Radial distortion (c)"))
     ADDCOLUMN(wxT("d"), "d", 40, wxALIGN_LEFT, true, HuginBase::ImageVariableGroup::IVE_RadialDistortionCenterShift, _("Horizontal image center shift (d)"))
     ADDCOLUMN(wxT("e"), "e", 40, wxALIGN_LEFT, true, HuginBase::ImageVariableGroup::IVE_RadialDistortionCenterShift, _("Vertical image center shift (e)"))
     ADDCOLUMN(wxT("g"), "g", 40, wxALIGN_LEFT, true, HuginBase::ImageVariableGroup::IVE_Shear, _("Horizontal image shearing (g)"))
@@ -373,10 +377,10 @@ void ImagesTreeCtrl::UpdateImageText(wxTreeItemId item
     // NAMEisLinked returns always false, if the lens or stacks contains only a single image
     // so add a special handling for this case
     bool isSingleImage = false;
-    if (m_groupMode != GROUP_NONE)
-    {
-        isSingleImage = (GetChildrenCount(GetItemParent(item)) == 1);
-    };
+    //if (m_groupMode != GROUP_NONE)
+    //{
+    //    isSingleImage = (GetChildrenCount(GetItemParent(item)) == 1);
+    //};
     wxString s;
     const size_t imgNr=itemData->GetImgNr();
     const HuginBase::SrcPanoImage & img = m_pano->getImage(imgNr);
@@ -468,18 +472,32 @@ void ImagesTreeCtrl::UpdateImageText(wxTreeItemId item
         SetItemText(item, m_columnMap["v"], hugin_utils::doubleTowxString(img.getHFOV(), m_degDigits));
     };
 
-    if (m_groupMode == GROUP_LENS && (img.RadialDistortionisLinked() || isSingleImage))
+    if (m_groupMode == GROUP_LENS && (img.RadialDistortionAisLinked() || isSingleImage))
     {
         SetItemText(item, m_columnMap["a"], wxEmptyString);
+    }
+    else
+    {
+        double dist=img.getRadialDistortionA();
+        SetItemText(item, m_columnMap["a"], hugin_utils::doubleTowxString(dist,m_distDigits));
+    };
+    if (m_groupMode == GROUP_LENS && (img.RadialDistortionBisLinked() || isSingleImage))
+    {
         SetItemText(item, m_columnMap["b"], wxEmptyString);
+    }
+    else
+    {
+        double dist=img.getRadialDistortionB();
+        SetItemText(item, m_columnMap["b"], hugin_utils::doubleTowxString(dist,m_distDigits));
+    };
+    if (m_groupMode == GROUP_LENS && (img.RadialDistortionCisLinked() || isSingleImage))
+    {
         SetItemText(item, m_columnMap["c"], wxEmptyString);
     }
     else
     {
-        std::vector<double> dist=img.getRadialDistortion();
-        SetItemText(item, m_columnMap["a"], hugin_utils::doubleTowxString(dist[0],m_distDigits));
-        SetItemText(item, m_columnMap["b"], hugin_utils::doubleTowxString(dist[1],m_distDigits));
-        SetItemText(item, m_columnMap["c"], hugin_utils::doubleTowxString(dist[2],m_distDigits));
+        double dist=img.getRadialDistortionC();
+        SetItemText(item, m_columnMap["c"], hugin_utils::doubleTowxString(dist,m_distDigits));
     };
 
     if (m_groupMode == GROUP_LENS && (img.RadialDistortionCenterShiftisLinked() || isSingleImage))
@@ -602,10 +620,10 @@ void ImagesTreeCtrl::UpdateGroupText(wxTreeItemId item
     // NAMEisLinked returns always false, if the lens or stacks contains only a single image
     // so add a special handling for this case
     bool haveSingleChild = false;
-    if (m_groupMode != GROUP_NONE)
-    {
-        haveSingleChild = (GetChildrenCount(item, false) == 1);
-    };
+    //if (m_groupMode != GROUP_NONE)
+    //{
+    //    haveSingleChild = (GetChildrenCount(item, false) == 1);
+    //};
     if(childItem.IsOk())
     {
         ImagesTreeData* data=static_cast<ImagesTreeData*>(GetItemData(childItem));
@@ -662,17 +680,31 @@ void ImagesTreeCtrl::UpdateGroupText(wxTreeItemId item
             SetItemText(item, m_columnMap["v"], wxEmptyString);
         };
 
-        if (m_groupMode == GROUP_LENS && (img.RadialDistortionisLinked() || haveSingleChild))
+        if (m_groupMode == GROUP_LENS && (img.RadialDistortionAisLinked() || haveSingleChild))
         {
-            std::vector<double> dist=img.getRadialDistortion();
-            SetItemText(item, m_columnMap["a"], hugin_utils::doubleTowxString(dist[0],m_distDigits));
-            SetItemText(item, m_columnMap["b"], hugin_utils::doubleTowxString(dist[1],m_distDigits));
-            SetItemText(item, m_columnMap["c"], hugin_utils::doubleTowxString(dist[2],m_distDigits));
+            double dist=img.getRadialDistortionA();
+            SetItemText(item, m_columnMap["a"], hugin_utils::doubleTowxString(dist,m_distDigits));
         }
         else
         {
             SetItemText(item, m_columnMap["a"], wxEmptyString);
+        };
+        if (m_groupMode == GROUP_LENS && (img.RadialDistortionBisLinked() || haveSingleChild))
+        {
+            double dist=img.getRadialDistortionB();
+            SetItemText(item, m_columnMap["b"], hugin_utils::doubleTowxString(dist,m_distDigits));
+        }
+        else
+        {
             SetItemText(item, m_columnMap["b"], wxEmptyString);
+        };
+        if (m_groupMode == GROUP_LENS && (img.RadialDistortionCisLinked() || haveSingleChild))
+        {
+            double dist=img.getRadialDistortionC();
+            SetItemText(item, m_columnMap["c"], hugin_utils::doubleTowxString(dist,m_distDigits));
+        }
+        else
+        {
             SetItemText(item, m_columnMap["c"], wxEmptyString);
         };
 
@@ -1147,8 +1179,14 @@ void ImagesTreeCtrl::OnContextMenu(wxTreeEvent & e)
             };
             if(m_optimizerMode)
             {
-                if(data->IsGroup() == emptyText)
+                bool isGroup=data->IsGroup();
+                if(!isGroup && !emptyText)
                 {
+                    menu.Append(ID_SELECT_SELECTED, _("Select"));
+                    menu.Append(ID_UNSELECT_SELECTED, _("Unselect"));
+                };
+                if(isGroup == emptyText)
+                {
                     if(m_groupMode==GROUP_LENS && m_variableVector[m_selectedColumn]!=HuginBase::ImageVariableGroup::IVE_Yaw)
                     {
                         menu.Append(ID_SELECT_LENS_STACK, _("Select all for current lens"));
@@ -1815,7 +1853,7 @@ void ImagesTreeCtrl::OnMouseMove(wxMouseEvent &e)
     e.Skip();
 };
 
-void ImagesTreeCtrl::SelectAllParameters(bool select, bool allImages)
+void ImagesTreeCtrl::SelectParameters(bool select, bool allImages, bool selectedImages)
 {
     std::set<std::string> imgVars;
     std::string var=m_columnVector[m_selectedColumn];
@@ -1857,6 +1895,10 @@ void ImagesTreeCtrl::SelectAllParameters(bool select, 
     {
         fill_set(imgs, 0, m_pano->getNrOfImages()-1);
     }
+    else if(selectedImages)
+    {
+        imgs=GetSelectedImages();
+    }
     else
     {
         wxArrayTreeItemIds selectedItem;
@@ -1929,22 +1971,32 @@ void ImagesTreeCtrl::SelectAllParameters(bool select, 
 
 void ImagesTreeCtrl::OnSelectAll(wxCommandEvent &e)
 {
-    SelectAllParameters(true, true);
+    SelectParameters(true, true, false);
 };
 
 void ImagesTreeCtrl::OnUnselectAll(wxCommandEvent &e)
 {
-    SelectAllParameters(false, true);
+    SelectParameters(false, true, false);
 };
 
 void ImagesTreeCtrl::OnSelectLensStack(wxCommandEvent &e)
 {
-    SelectAllParameters(true, false);
+    SelectParameters(true, false, false);
 };
 
 void ImagesTreeCtrl::OnUnselectLensStack(wxCommandEvent &e)
 {
-    SelectAllParameters(false, false);
+    SelectParameters(false, false, false);
+};
+
+void ImagesTreeCtrl::OnSelectSelected(wxCommandEvent &e)
+{
+    SelectParameters(true, false, true);
+};
+
+void ImagesTreeCtrl::OnUnselectSelected(wxCommandEvent &e)
+{
+    SelectParameters(false, false, true);
 };
 
 void ImagesTreeCtrl::OnChar(wxTreeEvent &e)
