--- src/hugin_cpfind/cpfind/PanoDetectorLogic.cpp.orig	2018-11-16 14:23:52 UTC
+++ src/hugin_cpfind/cpfind/PanoDetectorLogic.cpp
@@ -806,6 +806,33 @@ bool PanoDetector::MakeKeyPointDescriptorsInImage(ImgD
 
 bool PanoDetector::RemapBackKeypoints(ImgData& ioImgInfo, const PanoDetector& iPanoDetector)
 {
+    if (ioImgInfo.NeedsRemapping())
+    {
+        TRACE_IMG("Remapping back keypoints...");
+        HuginBase::PTools::Transform trafo1;
+        trafo1.createTransform(iPanoDetector._panoramaInfoCopy.getSrcImage(ioImgInfo._number),
+            ioImgInfo._projOpts);
+
+        int const dx1 = ioImgInfo._projOpts.getROI().left();
+        int const dy1 = ioImgInfo._projOpts.getROI().top();
+
+        double const scale = ioImgInfo.IsDownscale() ? 2.0 : 1.0;
+
+        for (size_t i = 0; i < ioImgInfo._kp.size(); ++i)
+        {
+            lfeat::KeyPointPtr& aK = ioImgInfo._kp[i];
+            double xout, yout;
+            if (trafo1.transformImgCoord(xout, yout, aK->_x + dx1, aK->_y + dy1))
+            {
+                // downscaling is take care of by the remapping transform
+                // no need for multiplying the scale factor...
+                aK->_x = xout;
+                aK->_y = yout;
+                aK->_scale *= scale;
+            };
+        };
+        return true;
+    };
     if (ioImgInfo.IsDownscale())
     {
         for (size_t i = 0; i < ioImgInfo._kp.size(); ++i)
@@ -816,32 +843,6 @@ bool PanoDetector::RemapBackKeypoints(ImgData& ioImgIn
             aK->_scale *= 2.0;
         };
     }
-    else
-    {
-        if (ioImgInfo.NeedsRemapping())
-        {
-            TRACE_IMG("Remapping back keypoints...");
-            HuginBase::PTools::Transform trafo1;
-            trafo1.createTransform(iPanoDetector._panoramaInfoCopy.getSrcImage(ioImgInfo._number),
-                ioImgInfo._projOpts);
-
-            int dx1 = ioImgInfo._projOpts.getROI().left();
-            int dy1 = ioImgInfo._projOpts.getROI().top();
-
-            for (size_t i = 0; i < ioImgInfo._kp.size(); ++i)
-            {
-                lfeat::KeyPointPtr& aK = ioImgInfo._kp[i];
-                double xout, yout;
-                if (trafo1.transformImgCoord(xout, yout, aK->_x + dx1, aK->_y + dy1))
-                {
-                    // downscaling is take care of by the remapping transform
-                    // no need for multiplying the scale factor...
-                    aK->_x = xout;
-                    aK->_y = yout;
-                };
-            };
-        };
-    };
     return true;
 }
 
